/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.shape;

import java.awt.Image;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import nl.cloudfarming.client.field.model.ShapeImportSource;
import nl.cloudfarming.client.imp.AbstractImportFolderNode;
import nl.cloudfarming.client.imp.AbstractImportProcessor;
import nl.cloudfarming.client.imp.ImportProcessor;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;

/**
 * Processor for ESRI shape files.
 *
 * <h2>Import</h2> <p> Import copies the provided file object and all secondary
 * files to the import folder. </p>
 *
 * <h2>Process</h2> <p> A shape file can also be processed. This means the
 * content of the shape file will be mapped to specific domain object. If not
 * enough information is present to process the shapefile, a dialog will be
 * initiated requesting the user to provide the missing information. </p>
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@NbBundle.Messages("shape import format description=ESRI shape file (*.shp)")
@ServiceProvider(service = ImportProcessor.class)
public class ShapeImportProcessor extends AbstractImportProcessor {

    public static final String IMPORTED_SHAPES_FOLDER = "Shapes";
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field.shape.ShapeImportProcessor");
    static final Icon ICON = ImageUtilities.loadImageIcon("nl/cloudfarming/client/field/shape/file_extension_shp.png", true);

    /**
     * {
     *
     * @inheritDoc }
     */
    @Override
    public String[] getSupportedExtentions() {
        return new String[]{ShapeFileLoader.PRIMARY_EXT};
    }

    /**
     * {
     *
     * @inheritDoc }
     */
    @Override
    public boolean isSupported(FileObject fo) {
        return ShapeFileLoader.PRIMARY_EXT.equals(fo.getExt());
    }

    /**
     * {
     *
     * @inheritDoc }
     */
    @Override
    public String getFormatDescription() {
        return Bundle.shape_import_format_description();
    }

    /**
     * {
     *
     * @inheritDoc }
     */
    @Override
    public Icon getIcon() {
        return ICON;
    }

    /**
     * {
     *
     * @inheritDoc }
     */
    @Override
    public void doImport(FileObject fo, Project prjct) throws IllegalArgumentException {
        assert isSupported(fo) : "unsupported FileObject passed to doImport, check for support first";
        try {
            DataObject dataObject = DataObject.find(fo);
            copySourceFiles(dataObject, getShapeImportFolder(prjct));
        } catch (IOException ex) {
            LOGGER.log(Level.WARNING, "Unable to find dataobject for shape fileobject {0}", ex.getMessage());
        }
    }

    /**
     * {
     *
     * @inheritDoc }
     */
    @Override
    public void process(FileObject fo, Project prjct) throws IllegalArgumentException {
        //TODO implement process for shape files.
        throw new UnsupportedOperationException("Not supported yet.");
    }

    /**
     * {
     *
     * @inheritDoc }
     */
    @Override
    public <T> void process(FileObject fo, Project prjct, List<T> toPopulate) throws IllegalArgumentException {
        //TODO implement process for shape files.
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public Node getImportNode(Project project) {
        try {
            FileObject shapesImportFolder = getShapeImportFolder(project);
            DataFolder dataFolder = DataFolder.findFolder(shapesImportFolder);
            assert dataFolder != null;
            return new ImportedShapesNode(project, dataFolder.getNodeDelegate());

        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "Could not find or create folder {0} with the exception {1}", new Object[]{IMPORTED_SHAPES_FOLDER, ex});
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    private FileObject getShapeImportFolder(Project project) throws IOException {
        return getSubFolder(getImportRootFolder(project), IMPORTED_SHAPES_FOLDER);
    }

    private static final class ImportedShapesNode extends AbstractImportFolderNode {
        public ImportedShapesNode(Project project, Node original) {
            super(project, original, new ImportedShapesNodeChildren(original));

        }

        @Override
        public Image getIcon(int type) {
            return ImageUtilities.loadImage("nl/cloudfarming/client/field/shape/folder_shapes16.png", true);
        }
    }

    public static class ImportedShapesNodeChildren extends FilterNode.Children {

        public ImportedShapesNodeChildren(Node node) {
            super(node);
        }

        @Override
        protected Node[] createNodes(Node object) {
            List<Node> result = new ArrayList<>();

            for (Node node : super.createNodes(object)) {
                if (accept(node)) {
                    result.add(node);
                }
            }
            return result.toArray(new Node[0]);
        }

        private boolean accept(Node node) {
            // filter out secondary files:
            ShapeImportSource sis = null;
            // It is not adviced to lookup a dataobject on a node
            // (see http://netbeans.org/bugzilla/show_bug.cgi?id=199391)
            // instead we lookup the fileobject and load the dataobject
            // from the fileobject.
            // if the dataobject contains a shape import source in its lookup
            // it is the primary object and should be shown.
            // secondary files are filtered this way
            FileObject fo = node.getLookup().lookup(FileObject.class);
            if(fo != null){
                try {
                    DataObject dataObject = DataObject.find(fo);
                    if (dataObject != null) {
                        sis = (ShapeImportSource) dataObject.getLookup().lookup(ShapeImportSource.class);
                    }
                } catch (DataObjectNotFoundException ex) {
                    //rejecting
                }
            }
            return (sis != null);
        }
    }

}
